# -*- coding: utf-8 -*- 
#  -------------------------------------------------------------------
#  --- SGLIsample_20210107a.py ---
#  -------------------------------------------------------------------
import sys
import os
import datetime
import numpy as np
import h5py
import cv2

Proc1st=datetime.datetime.now()
print("\n\n", " ... SGLIsample from ",Proc1st)

#  --- Target Info ---
filename='X:\\GC1SG1_202104100117C04810_1BSG_VNRDQ_2002.h5'
imggrp='Image_data'
sdname='Lt_VN05'
l1b_png=sdname+'.png'
map_png=sdname+'_MAP.png'
map_tif=sdname+'_MAP.tif'

max_ref=0.2
pngratio=0.001  # png resize ratio
degStep=0.002   # 0.002deg grid

print('target hdf :', filename)
print('target sd  :', imggrp, sdname)

#  open hdf / get sd
h5_file=h5py.File(filename, 'r')
sd=h5_file[imggrp][sdname]
(lines, pixels)=sd.shape

# Attribute
Offset=sd.attrs['Offset']
Slope=sd.attrs['Slope']
MaxDN=sd.attrs['Maximum_valid_DN']
MinDN=sd.attrs['Minimum_valid_DN']
Mask=sd.attrs['Mask']
TOA=sd.attrs['Band_weighted_TOA_solar_irradiance']

#  lat, lon
lat=h5_file['Geometry_data/' + 'Latitude']
lon=h5_file['Geometry_data/' + 'Longitude']
(latlon_lines, latlon_pixels) = lat.shape
respl=10    # resampling = 10

print('pixels={:}, lines={:}'.format(pixels, lines))
print('lat, lon : pixels={:}, lines={:}'.format(latlon_pixels, latlon_lines))
print('Slope={:}/Offset={:}/MinDN={:}/MaxDN={:}/TOA={:}'.format(Slope,Offset,MinDN,MaxDN,TOA))

#  engineering conversion
sd=sd[:] & Mask
img=sd.astype('float32')
img[np.where(img>MaxDN)]=np.nan
img[np.where(img<MinDN)]=np.nan
img=(img*Slope+Offset)/TOA
del sd

# save png before projection
dsp=img.reshape(lines,pixels,1)/max_ref*255.0
dsp=cv2.applyColorMap(dsp.astype('uint8'),cv2.COLORMAP_JET)
dsize=(int(lines*pngratio), int(pixels*pngratio))
cv2.resize(dsp,dsize)
cv2.imwrite(l1b_png,dsp)
del dsp

#  lat, lon range -> projection Mat
print('lat.shape,lat.dtype : ',lat.shape,lat.dtype)
(minLat, maxLat) = (np.nanmin(lat), np.nanmax(lat))
(minLon, maxLon) = (np.nanmin(lon), np.nanmax(lon))
(dst_lins,dst_pixs)=(int((maxLat-minLat)/degStep+0.5), int((maxLon-minLon)/degStep+0.5))

print('proj. step : {:} [pix/deg]'.format(degStep))
print('proj. lon range : ({:.2f}:{:.2f}) -> {:} pixs'.format(minLon,maxLon,dst_pixs))
print('proj. lat range : ({:.2f}:{:.2f}) -> {:} lins'.format(minLat,maxLat,dst_lins))

# destination Mat
dst=np.empty((dst_lins,dst_pixs),dtype=np.float32)*np.nan

# projection Loop : 透視変換（射影変換）
print("\n... start of projection", end='')
for lin in range(0,lines-respl-2,respl):
    if np.mod(lin, 500)==0: 
        print('\nline-{:04d}/{:04d} '.format(lin,lines), end='')
    for pix in range(0,pixels-respl-2,respl):

        # source ULLR
        srcPnt=[]
        srcPnt.append([pix,      lin      ])  # UL
        srcPnt.append([pix,      lin+respl])  # LL
        srcPnt.append([pix+respl,lin+respl])  # LR
        srcPnt.append([pix+respl,lin      ])  # UR
        srcMat=img[lin:lin+respl+2, pix:pix+respl+2]   # underlap consideration

        # desination ULLR
        (pix0,lin0)=(int(pix/respl),int(lin/respl))
        (pix1,lin1)=(pix0+1,lin0+1)
        if pix1>=latlon_pixels or lin1>=latlon_lines: continue
        
        (ULlon,ULlat)=(lon[lin0,pix0], lat[lin0,pix0]) # UL
        (LLlon,LLlat)=(lon[lin1,pix0], lat[lin1,pix0]) # UL
        (LRlon,LRlat)=(lon[lin1,pix1], lat[lin1,pix1]) # UL
        (URlon,URlat)=(lon[lin0,pix1], lat[lin0,pix1]) # UL

        [ULlon,LLlon,LRlon,URlon]=([ULlon,LLlon,LRlon,URlon]-minLon)/degStep
        [ULlat,LLlat,LRlat,URlat]=(maxLat-[ULlat,LLlat,LRlat,URlat])/degStep

        dstPnt=[]
        dstPnt.append([ULlon , ULlat]) # UL(lon,lat)
        dstPnt.append([LLlon , LLlat]) # LL(lon,lat)
        dstPnt.append([LRlon , LRlat]) # LR(lon,lat)
        dstPnt.append([URlon , URlat]) # UR(lon,lat)

        # destination size
        maxLonTmp=np.max([ULlon, LLlon, LRlon, URlon])
        minLonTmp=np.min([ULlon, LLlon, LRlon, URlon])
        maxLatTmp=np.max([ULlat, LLlat, LRlat, URlat])
        minLatTmp=np.min([ULlat, LLlat, LRlat, URlat])
        tmp_pixs=int(maxLonTmp-minLonTmp+0.5)+2  # underlap consideration
        tmp_lins=int(maxLatTmp-minLatTmp+0.5)+2

        # relative addressing & projection matrix
        srcPnt[0][0]=srcPnt[0][0]-0.5
        srcPnt[0][1]=srcPnt[0][1]-0.5
        for i in range(3,0,-1):
            srcPnt[i]=[srcPnt[i][0]-srcPnt[0][0], srcPnt[i][1]-srcPnt[0][1]]
            dstPnt[i]=[dstPnt[i][0]-minLonTmp,    dstPnt[i][1]-minLatTmp   ]
        srcPnt[0]=[0,0]
        dstPnt[0]=[0,0]
        srcPnt=np.array(srcPnt,dtype=np.float32)
        dstPnt=np.array(dstPnt,dtype=np.float32)
        H=cv2.getPerspectiveTransform(srcPnt,dstPnt)

        # projection
        tmpMat=np.zeros((tmp_lins,tmp_pixs)).astype('float32')
        tmpMat[:]=np.nan
        dsize=(tmp_pixs, tmp_lins)
        tmpMat=cv2.warpPerspective(srcMat,H,dsize,dst=tmpMat,borderMode=cv2.BORDER_TRANSPARENT)

        # merge
        for tmp_lin in range(tmp_lins):
            dst_lin=tmp_lin+int(minLatTmp+0.5)
            for tmp_pix in range(tmp_pixs):
                dst_pix=tmp_pix+int(minLonTmp+0.5)
                if np.isnan(dst[dst_lin,dst_pix]):
                    dst[dst_lin,dst_pix]=tmpMat[tmp_lin,tmp_pix]

        # prepartaion for next loop
        del tmpMat
        del srcMat
        if pix==0: print('.', end='')

h5_file.close()
print("\n")

# save project png
dsp=dst/max_ref*255.0
dsp=cv2.applyColorMap(dsp.astype('uint8'),cv2.COLORMAP_JET)
dsize=(int(dst_lins*pngratio), int(dst_pixs*pngratio))
cv2.resize(dsp,dsize)
cv2.imwrite(map_png,dsp)
del dsp
print("\n... end of map png output")

# --- end of process ---
del img
print()

ProcEnd=datetime.datetime.now()
print(ProcEnd, "... end of SGLIsample : elapse time=", ProcEnd-Proc1st)
print("\n\n")
